package pinball.log;
 
import java.io.*;
import java.util.*;
import java.text.*;

/** 
 * This reads the header part of the log file and generates an html file with the statistics.
 */
public class HeaderReport {
	/** File that will get the data. */
	private PrintWriter output;
	
	/** The time the log file was first created. */
	private String startTime;
	
	/** Information to be written to the log. */
	private Vector headerInfo=new Vector();
	
	/**
	 * Creates the header reports for the log files.
	 *
	 * @param out the output file.
	 */
	public HeaderReport(PrintWriter out){
		output=out;
	}
	
	/**
	 * Analyzes the header part of the log file.
	 *
	 * @param data the command line from the log file.
	 */
	public void parse(String data) {
		StringTokenizer tokens;
		String command;
			
		
		tokens = new StringTokenizer(data,"=");
		
		command=tokens.nextToken();
		
		/* Get the time the log file was generated. */
		if(command.equals("startedAt"))
			startTime=tokens.nextToken();
			
		/* Get the name of the player for the system. */
		else if(command.equals("PLAYER"))
			output.println("<H1>Analysis for "+tokens.nextToken()+" player at "+startTime+"</H1>");
			
		/* Header commands. */
		else if(command.equals("imageX"))
			headerInfo.addElement(new HeaderInfo("Image Width",tokens.nextToken()));
		else if(command.equals("imageY"))
			headerInfo.addElement(new HeaderInfo("Image Height",tokens.nextToken()));
		else if(command.equals("brightness"))
			headerInfo.addElement(new HeaderInfo("Image Brightness",tokens.nextToken()));
		else if(command.equals("contrast"))
			headerInfo.addElement(new HeaderInfo("Image Contrast",tokens.nextToken()));
		else if(command.equals("pixelThreshold"))
			headerInfo.addElement(new HeaderInfo("Pixel Threshold",tokens.nextToken()));
		else if(command.equals("colorDepth"))
			headerInfo.addElement(new HeaderInfo("Color Depth",tokens.nextToken()));
		else if(command.equals("ballMin"))
			headerInfo.addElement(new HeaderInfo("Ball Min Size",tokens.nextToken()));
		else if(command.equals("ballMax"))
			headerInfo.addElement(new HeaderInfo("Ball Max Size",tokens.nextToken()));
		else if(command.equals("moveThreshold"))
			headerInfo.addElement(new HeaderInfo("Move Threshold",tokens.nextToken()));

		else if(command.equals("postThread"))
			headerInfo.addElement(new HeaderInfo("Post Thread",tokens.nextToken()));
		else if(command.equals("analyzeThread"))
			headerInfo.addElement(new HeaderInfo("Analyze Thread",tokens.nextToken()));
		else if(command.equals("strategyThread"))
			headerInfo.addElement(new HeaderInfo("Strategy Thread",tokens.nextToken()));

		else if(command.equals("recording"))
			headerInfo.addElement(new HeaderInfo("Recording Game",tokens.nextToken()));
		else if(command.equals("playing"))
			headerInfo.addElement(new HeaderInfo("Playing Game",tokens.nextToken()));
			
		else if(command.equals("pixelRate"))
			headerInfo.addElement(new HeaderInfo("Pixel Rate",tokens.nextToken()));
		else if(command.equals("frameRate"))
			headerInfo.addElement(new HeaderInfo("Frame Rate",tokens.nextToken()));
			
		else if(command.equals("flipperResetTime"))
			headerInfo.addElement(new HeaderInfo("Flipper Reset Time",tokens.nextToken()));


			

	}
	
	/** 
	 * Write the header information to the log file. 
	 */
	public void writeInfo() {
		HeaderInfo info;
		/* Table of header information. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		
		for(int i=0;i<headerInfo.size();i++) {
			info=(HeaderInfo)headerInfo.elementAt(i);
			output.println("\t<TR>");
			output.println("\t\t<TD>"+info.getInfo()+"</TD>");
			output.println("\t\t<TD>"+info.getValue()+"</TD>");
			output.println("\t</TR>");
		}

		output.println("</TABLE><BR>");
	}

}
