package pinball.log;


import java.io.*;
import java.util.*;
import java.text.*;

/**
 * Gets the files used in the log file. 
 */
public class LogFileAccess{
	/** File that data the data. */	
	private BufferedReader input;
	
	/** Time and date part of the filename. */
	private String logTimeDate;
	
	/** Ball number for log file. */
	private int ballNumber;
	
	/** Description part of the log file. */
   	private String logDescription;

	/**
	 * Opens up the log files in their file order.
	 *
	 * @param start the name of the first log file.
	 */
	public LogFileAccess (String start) {
		System.err.println("Initial Log File: "+start);
		
		/* Separate the file name into it's parts. */
		StringTokenizer	st=new StringTokenizer(start,"_");		
		logTimeDate=st.nextToken();
		ballNumber=new Integer(st.nextToken()).intValue();

		/* Set the ball to be one less than the actual number, 
		 * because it's number is incremented when in use. 
		 */
		ballNumber--;
		
		logDescription=st.nextToken(" ");
	}

	
	/**
	 * Get the log file name.
	 *
	 * @return the log file name.
	 */
	public String getLogName() {
		return new String(logTimeDate+"_"+ballNumber()+logDescription);
	}
	
	/** 
	 * Set the ball number. 
	 *
	 * @return the ball's number.
	 */
	private String ballNumber() {
		if(ballNumber<10)
			return new String("00"+ballNumber);
		else if(ballNumber<100)
			return new String("0"+ballNumber);
		else return new String(""+ballNumber);
	}
	
	/**
	 * Get the next log file. 
	 *
	 * @return the next log file.
	 */
	public BufferedReader getNext() throws IOException {

		/* Go to the next ball. */
		ballNumber++;
		
		input=new LogFile(getLogName(),'r').getReader();


		return input;
	}
}
