package pinball.log;

import pinball.buffer.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Used to display the frame from the log file.
 */
public class ViewerCanvas extends Canvas{

	/** The image from the camera. */
	protected 	Image myImage;

	/** The greyscale values for the painting of the image.*/
	protected	ColorModel model;
	
	/** The x size of the canvas (image). */
	protected	int width;
	
	/** The y size of the canvas (image). */	
	protected	int height;

	/** Where to draw to off screen.*/
	protected Graphics graphicsContext;
	
	/** The image to draw to. */
	protected Image  frameBuffer;
	

	/** 
	 * Initalize with the size and create the color table. 
	 *
	 * @param w width of the image.
	 * @param h height of the image.
	 */
	public ViewerCanvas(int w, int h){
		width=w;
		height=h;
		
		System.err.println("Canvas size: "+width+" "+height);
		
	       /* Create the image's color table. (Grey scale)*/
        	byte grey[] = new byte[64];
	        for (int i = 0; i < 64; i++) {
        	   grey[i] = (byte) (i*4);
	        }
		/* Pure white. */
		grey[63]= (byte) 255;
	      	model = new IndexColorModel(6, 64, grey, grey, grey);				
	      	
	}

	/**
	 * Used in the extended classed. 
	 */
	public ViewerCanvas() {}

	/**
	 * Make the off screen image to draw to. 
	 */
	public void makeImage() {
		/* Create frameBuffer. */
	        frameBuffer = createImage(width,height);

		/* Create off screen graphics context. */
		graphicsContext = frameBuffer.getGraphics();
	}


	/**
	 * Update the image's data. Draw to the off screen image.
	 *
	 * @param frame the camera's frame.
	 * @param x the ball's x location.
	 * @param y the ball's y location.
	 */
	public void updateFrame (CameraFrame frame, int x, int y) {
		byte imageArray[]=null;
		int width, height, xoffset, yoffset;
		
		/* Get the parts of the image. */
		imageArray=frame.getFrame();
		 		
		/* Get the image dimentions. */
		width=frame.getWidth();
		height=frame.getHeight();
		xoffset=frame.getXOffset();
		yoffset=frame.getYOffset();
	
		
		/* Make the image.*/
	        MemoryImageSource mis = 
        		new MemoryImageSource(width,height,
  		   	   		model, imageArray, 0, width);
	        myImage = createImage(mis);
		
		/* Draw the image of the game. */

		graphicsContext.drawImage(myImage,xoffset, yoffset, width, height,this);				
		
		/* Draw the cross for the ball's location. */
		if(x>-1 && y>-1) {			
			graphicsContext.setColor(Color.yellow);
			graphicsContext.drawLine(x,y-5,x,y+5);
			graphicsContext.drawLine(x+5,y,x-5,y);
		}

		/* Draw the image. */
		paint(getGraphics());
	}


	/**
	 * Draw the images. 
	 */
	public void paint(Graphics g) {        		                	               					
        	/* display the image on the screen. */
  		try{g.drawImage(frameBuffer,0,0,width,height,this);}
       		catch (Exception e) {}
        }

	/**
	 * Called when the window is updated. 
	 */
        public void update(Graphics g) {
	
		paint(g);

	}
	
	
	/** 
	 * Method used in creating the canvas. Sets up the size.
	 */
	public Dimension getPreferredSize(){
		return getMinimumSize();
	}
	
  	/** 
	 * Method used in creating the canvas. Sets up the size.
	 */
	public Dimension getMinimumSize() {
		return new Dimension(width,height);
	}
}
