/*

cqc.h Connectix QuickCam hardware interface.

Copyright (C) 1996, 1997 Alex Belits

This software contains ideas and modified parts from qcam,
written by Scott Laird.

Copyright (C) 1996 by Scott Laird

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL SCOTT LAIRD OR ALEX BELITS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#define QCPORT0                      0x3bc
#define QCPORT1                      0x378
#define QCPORT2                      0x278

#define DEFAULTCONFIGFILENAME        "/usr/local/etc/qcam.conf"

#define SCAN_PRIORITY                -20

#define QCAM_ALARM_TIME              3

#define QCAM_DEFAULT_SPEED           2

#define QCAM_VER_COLOR               0x10

/* Commands */

#define QCAM_CMD_SENDVFRAME          7  /*0x07*/
#define QCAM_CMD_SETBRIGHTNESS       11 /*0x0b*/
#define QCAM_CMD_SETTOP              13 /*0x0d*/
#define QCAM_CMD_SETLEFT             15 /*0x0f*/
#define QCAM_CMD_SETNUMV             17 /*0x11*/
#define QCAM_CMD_SETNUMH             19 /*0x13*/
#define QCAM_CMD_SENDVERSION         23 /*0x17*/
#define QCAM_CMD_BW_SETCONTRAST      25 /*0x19*/
#define QCAM_CMD_BW_AUTOADJUSTOFFSET 27 /*0x1b*/
#define QCAM_CMD_SETBLACK            29 /*0x0d*/
#define QCAM_CMD_SETWHITE            31 /*0x1f*/
#define QCAM_CMD_COLOR_SETHUE        33 /*0x21*/
#define QCAM_CMD_BW_GETOFFSET        33 /*0x21*/
#define QCAM_CMD_SETSATURATION       35 /*0x23*/
#define QCAM_CMD_COLOR_SETCONTRAST   37 /*0x25*/
#define QCAM_CMD_SENDSTATUS          41 /*0x29*/
#define QCAM_CMD_COLOR_SETSPEED      45 /*0x2d*/


/* Camera definition (NOT the same as in qcam) */

struct qcread_qcam{
unsigned int x;           /* horizontal image offset */
unsigned int y;           /* vertical image offset */
unsigned int xsize;       /* image width */
unsigned int ysize;       /* image height */
unsigned int decim;       /* image decimation */
unsigned int direction;   /* image direction A - 0, > - 1, V - 2, < - 3 */
unsigned int bpp;         /* bits per pixel */
unsigned int framesize;   /* frame size */
unsigned char *frame;     /* output frame */
unsigned char **rows;     /* rows pointers list */
unsigned char brightness; /* brightness */
unsigned char contrast;   /* contrast */
unsigned char hue;        /* hue */
int p0;                   /* ports */
int p1;
int p2;
int lockfile;             /* lock file descriptor */
int ioporthandle;         /* i/o port handle (if used) */
unsigned int camv;        /* version */
unsigned int color;       /* color flag */
unsigned int connv;       /* connector version */
unsigned int bidir;       /* bidirectional port */
unsigned int btpp;        /* bytes per pixel */
int alarmflag;            /* alarm flag */
unsigned char black;      /* black level */
unsigned char white;      /* white level */
unsigned char blackoffset;/* black offset */
unsigned char saturation; /* saturation */
unsigned char *frame1;    /* internal frame */
int zoom;		  /* Zoom */
int buffers;		  /* Amount of display buffers. */
int yield;		  /* Amount of pixels to display before giving up the
			     process. */
int display_rate;	  /* Display every nth frame. */			     

unsigned char vparam;     /* hardware mode for the camera */
};

struct qcread_qcam *qccreate(void);
/* Create camera definition */

void qcdelete(struct qcread_qcam *qc);
/* Delete camera definition */

inline int qcsend(const struct qcread_qcam *qc,unsigned char arg);
/* Send request */

inline unsigned char qcrecv(const struct qcread_qcam *qc);
/* Receive response */

inline int qccmd(const struct qcread_qcam *qc,unsigned char cmd,unsigned char param);
/* Send command */

inline unsigned int qcget(const struct qcread_qcam *qc,unsigned char cmd);
/* Get parameter */

int qcinit(struct qcread_qcam *qc,int portnum);
/* Initialize camera */

int qcsetimage(struct qcread_qcam *qc,unsigned char brightness,
unsigned char contrast,unsigned char saturation,
unsigned char hue,unsigned char black,unsigned char white,
unsigned char blackoffset);
/* Set image parameters */

int qchwframe(struct qcread_qcam *qc,int x,int y,int xsize,int ysize,int direction,
int bpp,int decim);
/* Set frame parameters */

int qcsethw(const struct qcread_qcam *qc);
/* Set parameters after initial setup */

void qcwaitforready(const struct qcread_qcam *qc);
/* Wait fro camera to become ready */

int qcreadframe(struct qcread_qcam *qc);
/* Read frame */

int qcinitset(struct qcread_qcam *qc);
/* Iinitialize after filling the descriptor from the config file */

int qcreadconfigfile(struct qcread_qcam *qc,char *filename,
char *cameraname,int quiet);
/* Read the config file */
