/*
	PThread's flippers
*/
#include    <stdio.h>
#include	<pthread.h>
#include	"interface.h"	/* interface for labpc digital i/o */
#include	"flipper.h"

/* the structure for the flipper. */
typedef	struct {
	u_int		delay;	/* how long to keep the flipper up. */
	const	WHICH_FLIPPER	which_flipper;
} FLIPPER_FUNC_STRUCT;

/* The threads for the flippers. */
static	pthread_t	flipper_thread[2];
static	pthread_cond_t	flipper_cond[] = {PTHREAD_COND_INITIALIZER,
		PTHREAD_COND_INITIALIZER};
static	pthread_mutex_t	flipper_mutex[] = {PTHREAD_MUTEX_INITIALIZER,
	PTHREAD_MUTEX_INITIALIZER};

/* The flippers. */
FLIPPER_FUNC_STRUCT	left_flipper_struct = {150000, LEFT_FLIPPER};
FLIPPER_FUNC_STRUCT	right_flipper_struct = {150000, RIGHT_FLIPPER};


/* flipper_func: flips the flipper that is passed to it is raised and
held for the delay specified and then lowered. */

static void	*flipper_func(void *arg)
{
	FLIPPER_FUNC_STRUCT	*a; /* The flipper function. */
	int	which,  /* Which flipper to flip and used to get the lock. */
		which2; /* Which part of PC Lab to use. */

	/* Get the flipper function. */
	a = (FLIPPER_FUNC_STRUCT *) arg;
	which = a->which_flipper; /* the flipper to flip. */
	
	/* Use the left or right part of labpc depending on if it's the left or 
	right flipper. */
	which2 = (which == LEFT_FLIPPER) ? LABPC_LEFT : LABPC_RIGHT;
	
	/* aquire the flipper lock. */
	pthread_mutex_lock(&flipper_mutex[which]);
	
	/* Go forever. Flip the designated flipper whenever it is signaled. */
	while (1) {
		pthread_cond_wait(&flipper_cond[which],
			&flipper_mutex[which]);
		fprintf(stderr, "Flip %s flipper.\n",
                (which == LEFT_FLIPPER) ? "left" : "right");
		
		/* send to the designated flipper a 1 to make it go up. */
		labpc_write(which2, 1);

		/* give a delay. */
		usleep(a->delay);

		/* send to the designated flipper a 0 to make it go down. */
		labpc_write(which2, 0);
	}

	/* return the flipper lock.*/
	pthread_mutex_unlock(&flipper_mutex[which]);
}


/* flip: flips one of the flippers passed to it. */
void	flip(WHICH_FLIPPER which)
{
	pthread_mutex_lock(&flipper_mutex[which]);
	pthread_cond_signal(&flipper_cond[which]);
	pthread_mutex_unlock(&flipper_mutex[which]);
}


/* init_flippers: initalizes the flippers. */
void	init_flippers(void)
{
	pthread_attr_t	detached_attr;
	static	struct	sched_param	sparam;
	static	int	policy;


	labpc_open(); /* opens the labpc to send the flipper commands. */

	/* Create the threads for the flippers. */

	pthread_attr_init(&detached_attr);

	/* therefore the threads are not join-able
		ie. i dont care when they die */
	pthread_attr_setdetachstate(&detached_attr,
		PTHREAD_CREATE_DETACHED);
	pthread_getschedparam(pthread_self(), &policy, &sparam);
	sparam.sched_priority+=3;
	pthread_attr_setschedparam(&detached_attr, &sparam);

	pthread_create(&flipper_thread[LEFT_FLIPPER], &detached_attr,
		flipper_func,
		(void *) &left_flipper_struct);
	pthread_create(&flipper_thread[RIGHT_FLIPPER], &detached_attr,
		flipper_func,
		(void *) &right_flipper_struct);

	pthread_attr_destroy(&detached_attr);
}

/* destroy_flippers: closes the labpc, you are done with the flippers. */
void	destroy_flippers(void)
{
	labpc_close();
}

/* set_delay: sets the delay of the flipper passed to it for the given value. */
void	set_delay(WHICH_FLIPPER which, u_int delay1)
{
	switch (which) {
	case LEFT_FLIPPER:
		left_flipper_struct.delay = delay1;
	case RIGHT_FLIPPER:
		right_flipper_struct.delay = delay1;
	}
}
