/*
	labpc driver
	- with quickcam

	port a&b - output (vary between 1 and 0)

	pa0 and pb0
*/

#include	<pthread.h>

#include	<stdio.h>
#include	<sys/ioctl.h>
#include	<errno.h>
#include	<sys/dataacq.h>
#include	<fcntl.h>

#include	"interface.h"

static	int	fda, fdb;

static pthread_mutex_t	labpc_lock = PTHREAD_MUTEX_INITIALIZER;

static int	isAlreadyOpen = 0;

/*
ssize_t	read_with_lock(int fd, void *buf, size_t nbytes)
{
	ssize_t	retval;

	pthread_mutex_lock(&labpc_lock);
	retval = read(fd, buf, nbytes);
	pthread_mutex_unlock(&labpc_lock);
	return retval;
}
*/

/*
	used in labpc_write() to send a command to one of the flippers.
*/
static void	port_write_(int fd, u_char val)
{
	int	retval;

	retval = write(fd, (const void *) &val, (size_t) 1);
	if (retval == -1)
		perror("Error in write()");
}

/* port_init: initalize the ports. */
static void	port_init_(void)
{
	port_write_(fda, 0);
	port_write_(fdb, 0);
}

/*
	used in labpc_open()
*/
static int	port_open_(char *devname)
{
	int	fd;

	fd = open(devname, O_WRONLY);
	if (fd == -1) {
		perror("Error in port_open()");
		return 0;
	} else {
		printf("Device Name = (%s) File Descriptor = (%d)\n",
			devname, fd);
		return fd;
	}
}

/* labpc_open: sets up the labpc. */
void	labpc_open(void)
{
	pthread_mutex_lock(&labpc_lock);
	if (!isAlreadyOpen) {
		fda = port_open_("/dev/labpcdio0");
		fdb = port_open_("/dev/labpcdio1");
		port_init_();
		isAlreadyOpen = 1;
	}
	pthread_mutex_unlock(&labpc_lock);
}

/* labpc_write: send a value to one of the ports. */
void	labpc_write(int port, u_char val)
{
	pthread_mutex_lock(&labpc_lock);
	if (isAlreadyOpen) {
		switch (port) {
		case IO_PORTA:
			port_write_(fda, val);
			break;
		case IO_PORTB:
			port_write_(fdb, val);
			break;
		default:
			puts("labpc_write: error");
		}
	}
	pthread_mutex_unlock(&labpc_lock);
}

/* labpc_close: closes the labpc. */
void	labpc_close(void)
{
	pthread_mutex_lock(&labpc_lock);
	if (isAlreadyOpen) {
		port_init_();
		if (close(fda) == -1)
			perror("lab_close: Error");
		if (close(fdb) == -1)
			perror("lab_close: Error");
		isAlreadyOpen = 0;
	}
	pthread_mutex_unlock(&labpc_lock);
}
