package pinball.part;

import pinball.*;
import pinball.io.*;
import pinball.buffer.*; 
import pinball.server.*;
import pinball.player.*;

import java.io.*;
import java.util.*;

/** 
 * The plunger for the pinball machine.
 */
public class Plunger {
	/** Used to control the plunger. */
	private static transient DataIO plunger;
	
	/** Code sent to lanuch the plunger. */
	private BitSet launch;
	
	/** Code to launch the ball. */
	private final int LAUNCHCODE=2;
	
	
	/** Has the plunger been launched. */
	private boolean launched=false;

	/** Create the plunger. */
	public Plunger(DataIO plunger){
		/* If playing from a file, don't make the plunger. */
		if(GameOptions.play())
			return;
		this.plunger=plunger;
		
		launch=new BitSet(plunger.WORDSIZE);
		
		launch.set(LAUNCHCODE);
		
		
		System.err.println("Plunger Made.");
	}
	
	/** 
	 * Launch the ball. 
	 */
	public void launchBall() {
		launched=true;
		GameOptions.launchBall(false);
		try{	plunger.setOutputLines(plunger.ZERO,launch);
			Thread.sleep(100);
			plunger.resetOutputLines(plunger.ZERO,launch);
		}
		catch (Exception e){}
	} 			
	
	/**
	 * Check if the ball was launched.
	 *
	 * @return launched status.
	 */
	public boolean ballLaunched() {
		return launched;
	}
	
	

	/** 
	 * Reset the plunger. 
	 */	
	public void reset() {
		launched=false;
		try {
			plunger.setOutputLines(plunger.ZERO,plunger.ZERO);
		}
		catch (Exception e){}
	}

}

