package pinball.player.adjustable;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * This is where the picture is displayed to.
 * It does the drawing of the image, ball location, flippers, flipper zone, in play and out
 * of play area.
 */
public class AdjustablePost extends Post {

	/* Area to draw to. */
	protected AdjustableDrawingCanvas canvas;
	

	/** 
	 * Initalize the DrawingBoard. 
	 *
	 * @param buffer the first in the series of infomation buffers.
	 */
	public AdjustablePost(Buffer buffer){	
		/* The first buffer. */
		this.buffer=buffer;
		
		setName("Post");	      	
	}
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "Post";
	}
	
	
	/**
	 * Create the canvas drawn to.
	 *
	 * @param x Width of canvas.
	 *
	 * @param y Height of canvas.
	 *
	 * @returns the canvas to drawn on.
	 */
	public DrawingCanvas canvas(int x,int y) {
		this.canvas=new AdjustableDrawingCanvas(x,y);
			      	
		return this.canvas;
	}
	/**
	 * Get the canvas drawn to.
	 *
	 * @returns the canvas drawn to.
	 */	
	public DrawingCanvas getCanvas() {
		return canvas;
	}

	/**
	 * Keep looking at the images and draw them at the frame rate from GameOptions. 
	 */
	public void run(){
			
		canvas.makeImage();

		buffer.lock().post(buffer.id());
		buffinfo=this.buffer.info();

		AdjustableBufferInfo adjustableBuffinfo = (AdjustableBufferInfo)buffinfo;
		
		AdjustableFlippers af=new AdjustableFlippers();

		adjustableBuffinfo.setAdjustableFlippers(af);
		canvas.setAdjustments(af);

		buffer.lock().posted(buffer.id());								
		buffer=buffer.next();					
		Thread.yield();

		/* Setup the stub in the DrawingCanvas. */
		setupStub();
		
		/* Keep drawing the images on the canvas. */	
		while(true) {

			buffer.lock().post(buffer.id());
			buffinfo=this.buffer.info();
						
			buffinfo.time().setStartDisplayTime();
			
			if(!buffer.skipped()) {
				/* Playing back the images from the file. Say where the ball is thought
  				   to be at and it was found to be at. 
  				*/
				if(GameOptions.play()) {
					System.err.println("Ball found at: "+buffinfo.blip().x()+","+buffinfo.blip().y());
					System.err.println("Ball thought to be found at: "+buffinfo.blip().foundX()+","+buffinfo.blip().foundY());
				}
						
				/* display the image, if it's the frame of the frame rate. */				
		        	if(buffinfo.number()%framerate==0) {
					/* Draw image. */
					if(GameOptions.debug()>=2)
						System.err.println("Frame #"+buffinfo.number());
				
					/* Draw the frame on the off screen image. */
					canvas.updateFrame(buffinfo); 
								
	
					if(GameOptions.debug()>=2)
				     		System.err.println("Going to paint.");
	
				}		
			}	
			buffinfo.time().setEndDisplayTime();

			buffer.lock().posted(buffer.id());						
			
			buffer=buffer.next();
			
			//skip();
		
			Thread.yield();
		}

	}



	/**
	 * Setup the stub in the drawing canvas. 
	 */
	public void setupStub() {
		buffer.lock().post(buffer.id());
		buffinfo=this.buffer.info();	
		
		canvas.setStub(buffinfo.getStub());
			
		buffer.lock().posted(buffer.id());						 	
		buffer=buffer.next();
	
		Thread.yield();
	}
		

	/** 
	 * Skip to the most recent buffer.
	 */
	private void skip() {
		int firstChecked=buffer.id();
				
		/* Not skippable. */
		if(buffer.lock().status()!=buffer.lock().strategied)
			return;			 
		
		/* Skip frames while the next buffer has a frame that's usable. */
		while((buffer.next().lock().status()==buffer.next().lock().strategied) && (buffer.next().id()!=firstChecked)) {

			/* Let this buffer pass through, since a later one is available. */

			/* Get the lock. */
			buffer.lock().post(buffer.id());
		
			/* Get the buffer info. */
			buffinfo=this.buffer.info();

			PinballLog.writeln(buffinfo.number(),"drawingBoardSkippingFrame="+buffer.info().number(),1,this);				
			System.err.println("DrawingBoard skipping frame #"+buffinfo.number());

			buffer.skipped(true);
			
			/* Release Lock. */
			buffer.lock().posted(buffer.id());
			
			buffer=buffer.next();
			
		}		
	}
}
