package pinball.player.adjustable;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;


/**
 * Player which can have the flipper options adjusted. 
 */
public class AdjustableStrategy extends Strategy {

	private AdjustableBufferInfo adjustableBuffinfo;

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system.
	 */		
	public AdjustableStrategy(Buffer buffer, TextArea gameStats,Camera camera) {

		this.stub=new Stub(buffer,gameStats,this,camera);
		id="Adjustable";
	}
 				
	/** For extended classes. */	
	public AdjustableStrategy() {

	}
				
	/**
	 * Should you flip or not. It never flips.
	 */
	public void decision() {
		adjustableBuffinfo = (AdjustableBufferInfo)buffinfo;
		
		/* Set the default that it did not flip. */	
		flipperFlipped(false);
		
		/* Is there a ball to flip? If so flip it.*/
		/* Add the offset value for the ball's location so it's
		correctly found from original location in game. */
		
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
					
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
			
		/* Check for valid movement. Make sure it's not the flipper that
		is seens as movement.*/
			
		/* Flip the flipper to the corresponding half of the frame
		where the ball is found. */
			
		/* Flip left flipper. */
		if(adjustableBuffinfo.getAdjustableFlippers().getLeft().contains(new Point(x,y))) {
			stub.leftFlipperState(true);
		}
		
		/* Flip right flipper. */
		if(adjustableBuffinfo.getAdjustableFlippers().getRight().contains(new Point(x,y))) {
			stub.rightFlipperState(true);
		}
		
	}
		

}
