package pinball.player.constant;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;

/**
 * The constant player's strategy. Constantly flips flippers. 
 */
public class ConstantStrategy extends Strategy {
	/** Time to rest between flips. */
	private int reset= GameOptions.resetTime();
	
	/** Did flip take place. */
	private boolean flip=false;		
	
	/** For extended classed. */
	public ConstantStrategy() {
	}
	
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */
	public ConstantStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.buffer=buffer;
		id="Constant";

		this.stub=new Stub(buffer,gameStats,this,camera);
		stub.setPriority(Thread.NORM_PRIORITY);
		
		/* Make a larger boarder around the flippers.
		   This helps the system locate the ball out of play.
		*/
		boarder=8;
		stub.boarder(boarder);		

	}
		
	
	/**
	 * Should you flip or not. This always flips.
	 */
	public void decision() {
		/* Reset the flippers. */
		if(flip==false) {
			stub.leftFlipperState(false);
			stub.rightFlipperState(false);
			flip=true;
		}

		/* Flip the flippers. */
		else if(flip==true) {
			stub.leftFlipperState(true);
			stub.rightFlipperState(true);
			flip=false;
		}
		
		/* Pause between flips. */		
		try {
      		       		Thread.sleep(reset);
     		} 
       		catch (InterruptedException e)
	       	{System.err.println(e);}
			
			
	}			
}
