package pinball.player.draw;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Draws the frame from the camera on the screen. 
 */
public class DrawDrawingCanvas extends DrawingCanvas{
	/** 
	 * Initalize the DrawingBoard. 
	 *
	 * @param w width of the image.
	 * @param h height of the image.
	 */
	public DrawDrawingCanvas(int w, int h){
		super(w,h);
	}


	/**
	 * Update the image's data. Draw to the off screen image.
	 *
	 * @param buffinfo the buffer information that has the image.
	 */
	public void updateFrame (BufferInfo buffinfo) {		
		/* The location of the ball. */
		int x=buffinfo.blip().x();
		int y=buffinfo.blip().y();
		
		/* Make the image.*/
	        MemoryImageSource mis = 
        		new MemoryImageSource(buffinfo.width(),buffinfo.height(),
  		   	   		model, buffinfo.get(), 0, buffinfo.width());

	        /* The image from the camera. */
		Image myImage = createImage(mis);

		if(GameOptions.debug()>=2)
	     		  System.err.println("Frame Updated.");
		
		/* Draw new image in the frameBuffer. */

		/* Clear the background if the buffer calls for it. 
		   Used when resizing to erase the background.*/
		if(buffinfo.clearing()) {
			Dimension d=getSize();
			graphicsContext.clearRect(0,0,d.width,d.height);
			buffinfo.clearing(false);
		}

		/* Draw the image of the game. */
		graphicsContext.drawImage(myImage,buffinfo.XOffset(),buffinfo.YOffset(),buffinfo.width(),buffinfo.height(),this);		
			
		
		/* Have the strategy draw it's lines so you can see the decision process. */
		if(stub!=null)
			stub.strategyDraw(graphicsContext,buffinfo.width(),buffinfo.height());		
		
		paint(getGraphics());	
	}


}
