package pinball.player.draw;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;


/** The player draws objects on the screen for the person to touch. */
public class DrawStrategy extends Strategy {

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public DrawStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		GameOptions.pixelThreshold(8);
		this.stub=new DrawStub(buffer,gameStats,this,camera);
		id="Draw";
	}
 				
				
	/**
	 * Should you flip or not. It never flips.
	 */
	public void decision() {
											
	}
	
	/**
	 * Draw some objects for the person touch. If they touch them, change the object's color.
	 *
	 * @param g the graphics to draw to.
	 * @param cavasWidth the width of the canvas.
	 * @param canvcasHeight the hight of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
		int boxW, boxH;
		
		boxW=canvasWidth/10;
		boxH=canvasHeight/3;
		
		Rectangle box1=new Rectangle(canvasWidth-canvasWidth/10,canvasHeight-canvasHeight/3,boxW,boxH);
		
		Rectangle box2=new Rectangle(canvasWidth-canvasWidth/10,0,boxW,boxH);
		
		g.setColor(Color.red);
		g.fill3DRect(box1.x,box1.y,box1.width,box1.height,true);

		g.setColor(Color.blue);
		g.fill3DRect(box2.x,box2.y,box2.width,box2.height,true);
			
		if(stub.blipx()!=-1 && stub.blipy()!=-1) {
		
			Rectangle hand=new Rectangle(canvasWidth-stub.blipx(),stub.blipy(),canvasWidth/20,canvasWidth/20);
			
			if(box1.intersects(hand)) {
				g.setColor(Color.green);
				g.fill3DRect(box1.x,box1.y,box1.width,box1.height,true);
			}				
				
			if(box2.intersects(hand)) {
				g.setColor(Color.cyan); 				
				g.fill3DRect(box2.x,box2.y,box2.width,box2.height,true);					
			}

			/* Hand pointer. */
			g.setColor(Color.yellow);
			g.fill3DRect(hand.x,hand.y,hand.width,hand.height,true);
		}
				

		
	}

}
