package pinball.player.holdhotspot;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.network.*;
import pinball.player.remote.*;

import java.awt.*;

/**
 * Player which hold the ball with the flippers before flipping. 
 * This player holds the flipper or flips depending on the location of the ball to the flipper.
 * If the ball is near the flipper it flips. If it's towards the extreme of the flipper, the flipper is held up to catch the ball.
 */
public class HoldHotSpotPlayer extends StdPlayer{
	/** Id of class. */
	static { id="HoldH"; }
	
	/** 
	 * Start up the program. Takes command line arguments.
   	 * Takes the following command line arguments:
	 * -s for server mode.
	 * -c for client mode.	 
	 */
	public static void main(String args[]) {		
		StdPlayer player=new HoldHotSpotPlayer();
		player.makeIt(player,args);
	} 
	
	
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy localStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new HoldHotSpotStrategy(buffer,gameStats,camera);
	}
	
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy clientStrategy(Buffer buffer,TextArea gameStats) {
		return new HoldHotSpotRemoteStrategy(buffer,gameStats);
	}
	

}
