package pinball.player.holdhotspot;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;


/** The player which HoldHotSpots the flippers up to catch the ball. */
public class HoldHotSpotStrategy extends Strategy {
	/** Distance from the flipper to be considered to hold the flipper up. */
	protected int holdDistance=10;

	/** Distance from the middles of the flippers flip both. */
	protected int middleDistance=3;


	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public HoldHotSpotStrategy(Buffer buffer, TextArea gameStats, Camera camera) {

		this.stub=new Stub(buffer,gameStats,this,camera);
		id="HoldHotSpot";
	}
 				
				
	public HoldHotSpotStrategy() {

	}
				
	/** 
	 * Should you flip or not. 
	 */
	public void decision() {		

		/* Set the default that it did not flip. */	
		flipperFlipped(false);
		
		/* Is there a ball to flip? If so flip it.*/
		/* Add the offset value for the ball's location so it's
		correctly found from original location in game. */
		
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
		
//		System.err.println("Ball: "+x+","+y+" Flipper height "+flipperHeight);
//		System.err.println("LeftX: "+leftX+" RightX: "+rightX);
			
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
			
		/* Flip the flippers. */
		//if(x>=0 && y>=flipperHeight) {
		if(x>0) {
		
			/* Check for valid movement. Make sure it's not the flipper that
			is seens as movement.*/
			
				/* Flip the flipper to the corresponding half of the frame
				where the ball is found. */
			
				/* Find the distance of the ball from the flippers, 
				   to see which to flip. If the ball's distance to a flipper is negative, 
				   the flipper is held up. If it has a closer positive value, then the
				   flipper is just raised and lowered.
				*/
				leftDistance=x-leftX;

				rightDistance=rightX-x;					
				
			
				/* Hold the left flipper up, if the ball is towards the left
				   of the flipper. 
				*/
				if(leftDistance<=-holdDistance) {
					stub.leftFlipperState(true);
				 }
				
				/* Flip the flipper, only if the ball is towards the right of the flipper and
				   has not passed the middle of the flippers area. 
				*/
				if(leftDistance>-holdDistance && leftDistance<=(middle-leftX)) {
					if(stub.isLeftDown())
						stub.leftFlipperState(true);
				 }
				
				/* Hold the right flipper up, if the ball is towards the right
				   of the flipper. 
				*/
				if(rightDistance<=-holdDistance) {
					stub.rightFlipperState(true);
				 }
				
				/* Flip the flipper, only if the ball is towards the left of the flipper and
				   has not passed the middle of the flippers area. 
				*/
				if(rightDistance>holdDistance && rightDistance<=(middle-rightX)) {
					if(stub.isRightDown())
						stub.rightFlipperState(true);
				 }

				/* Flip both. */
				if(x>(middle-middleDistance) && x<(middle+middleDistance)) {
					if(stub.isLeftDown())
						stub.leftFlipperState(true);
					if(stub.isRightDown())
						stub.rightFlipperState(true);					
				}			 
					 
		}

	}
	
	/**
	 * Allows the strategy to draw to the screen any lines or other graphics
	 * for the user to see how the strategy operates. In this strategy is shows the middle of the screen
	 * and the area that are used to hold the flippers up.
	 *
	 * @param g the graphics area to draw to.
	 * @param canvasWidth the width of the canvas.
	 * @param canvasHeight the height of the canvas.
	 */	
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
		g.setColor(Color.yellow);

		/* Middle of flippers. */
		g.drawLine(middle,0,middle,canvasHeight);
		
		/* The hold distance for the left flipper. */
		g.drawLine(leftX-holdDistance,0,leftX-holdDistance,canvasHeight);
	
		/* The hold distance for the right flipper. */
		g.drawLine(rightX+holdDistance,0,rightX+holdDistance,canvasHeight);

	}
	

}
