package pinball.player.human;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;


/**
 * Human player plays pinball. 
 */
public class HumanStrategy extends Strategy {	
	/** The window to draw to. */
	protected HumanWindow window;

	/** The command from the human to flip the flippers. */
	protected FlipperCommands commands;
	
	/** The previous flip status of the flippers. */
	protected boolean lastLeft=false, lastRight=false;

	/** Frame frame for requesting images. */
	protected int frameRate=1;
	
	/** Frame number. */
	protected int frame=1;
	
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
         * @param camera the camera for the system.
	 */		
	public HumanStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		this.stub=new Stub(buffer,gameStats,this,camera);
		id="human";

		Dimension imageSize=camera.imageSize();
		
		window=new HumanWindow(imageSize.width,imageSize.height,stub);
		commands=window.getFlipperCommands();
	}
 				
	/** For extended classes. */			
	public HumanStrategy() {

	}
	
	
	/**  
	 * Don't let the flippers reset on their own. The human player will reset them.
	 */ 
	public void flippersReset() {}
		
	/** Return the image from the stub. */
	public Image getImage() {
		return stub.getImage();
	}		
	
	/** Have the human presss the keys to raise of lower the flippers. */		
	public void decision() {

		
		/* Get the ball's location in the frame. */
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
	
		/* Add in the image offset to find the location	of the ball
		   in the area of the pinball machine.	*/
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
	
		/* Only request images at the frame rate. */		
		if(frame%frameRate==0) {
			stub.requestFrame();
		}
		
		Image image=stub.getImage();	
		if(image!=null)
			window.setImage(image);

		window.setBall(x,y);
		
			
		if(lastLeft!=commands.left()) {
			if(commands.left())
				stub.leftFlipperState(true);
			if(!commands.left())
				stub.leftFlipperState(false);
		}
		lastLeft=commands.left();

		if(lastRight!=commands.right()) {
			if(commands.right())
				stub.rightFlipperState(true);		
			if(!commands.right())
				stub.rightFlipperState(false);
		}
		lastRight=commands.right();
		
		window.doDraw();
			
		frame++;	
	}
	

}
