package pinball.player.human;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;

/** Displays the image from the pinball machine. Also captures the keystokes to play the pinball machine.*/
public class HumanWindow extends Frame{
	/** The strategy thread. */
	protected HumanStrategy strategy;
	
	/** The image from the camera. */
	protected Image image;
	
	/** Is the image drawable. */
	protected boolean drawable=false;
	
	/** Used for the key commands for the flippers. */
	protected FlipperCommands commands;
	
	/** Frame's dimensions. */
	protected int width, height;


	/** Where to draw to off screen.*/
	protected Graphics graphicsContext;
	
	/** The image to draw to. */
	protected Image  frameBuffer;
	
	/** The windows insets. */
	protected Insets insets;
	
	/** Location of the ball. */
	protected Point ball;
	

	/** Make the window.
	 * @param width the width of the window.
	 * @param height the height of the window.
	 * @param stub the stub from the system.
	 */
	public HumanWindow(int width, int height, Stub stub) {
	
		width*=2;
		height*=2;
		
		this.width=width;
		this.height=height;
		
		setSize(width,height);
		setVisible(true);
	
		insets=getInsets();		

		setSize(width+insets.left+insets.right,height+insets.top+insets.bottom);
			
		this.strategy=strategy;
		addKeyListener(commands=new FlipperCommands(stub));
		
		insets=getInsets();
		
		Dimension d=size();
		
		System.err.println("Human Window "+d.width+" "+d.height+" "+insets.left+" "+insets.top);		
		
		/* Create frameBuffer. */
	        frameBuffer = createImage(width,height);

		/* Create off screen graphics context. */
		graphicsContext = frameBuffer.getGraphics();
	}
	
	/** 
	 * Send the flipper comamnds.
	 *
	 * @return the flipper commands.
	 */
	 
	public FlipperCommands getFlipperCommands() {
		return commands;
	}
	
	/**
	 * Set the image from the camera.
	 *
	 * @param image The image to draw.
	 */
	public synchronized void setImage(Image image) {
		this.image=image;
	}
	
	/**
	 * Draw the image from the camera. 
	 */
		
	private synchronized void drawImage() {
		if(image==null)
			return;
		Dimension d=getSize();		
		graphicsContext.drawImage(image,(d.width-image.getWidth(this))/2,(d.height-image.getHeight(this))/2,this);
	}	

	/**
	 * Set the location of the ball. 
	 *
	 * @param x the x location.
	 * @param y the y location.
	 */	
	public synchronized void setBall(int x, int y) {
		ball=new Point(x,y);
	}
	
	/** 
	 * The the ball. 
	 *
	 * @param x the x location.
	 * @param y the y location.
	 */	
	private synchronized void drawBall(int x, int y) {
		/* Draw the cross for the ball's location. */
		if(x>-1 && y>-1 && image!=null) {
			/* The the location lines up with the image. */
			x+=(width-image.getWidth(this))/2;
			y+=(height-image.getHeight(this))/2;
			
			graphicsContext.setColor(Color.yellow);
			graphicsContext.drawLine(x,y-5,x,y+5);
			graphicsContext.drawLine(x+5,y,x-5,y);
		}

	}

	/** Draw the image. 
	*/
	public void update(Graphics g) {
		paint(g);		
	}

	/** Draw the image. 
	*/
	public synchronized void paint(Graphics g) {
		if(!drawable)
			return;
		if(image!=null) {			
			g.drawImage(frameBuffer,0,0,this);			
		}
		drawable=false;
	}
	
	/** Clear the window. */
	public synchronized void clear() {
		graphicsContext.clearRect(0,0,width,height);
	}
	
	/** Do the drawing to the window. */
	public void doDraw() {
		drawImage();
		drawBall(ball.x,ball.y);
		drawable=true;
		repaint();
		while(drawable) {
			try{Thread.sleep(10);}
			catch(Exception e){}
		}
		clear();
	}
			
		
}
